@@ -32,21 +32,38 @@ import sys
 from abc import ABC
 from collections import OrderedDict as _default_dict
 from collections.abc import MutableMapping
-from configparser import (ConfigParser, DuplicateOptionError,
-                          DuplicateSectionError, Error,
-                          MissingSectionHeaderError, NoOptionError,
-                          NoSectionError, ParsingError)
+from configparser import (
+    ConfigParser,
+    DuplicateOptionError,
+    DuplicateSectionError,
+    Error,
+    MissingSectionHeaderError,
+    NoOptionError,
+    NoSectionError,
+    ParsingError,
+)
 
-__all__ = ["NoSectionError", "DuplicateOptionError", "DuplicateSectionError",
-           "NoOptionError", "NoConfigFileReadError", "ParsingError",
-           "MissingSectionHeaderError", "ConfigUpdater"]
+__all__ = [
+    "NoSectionError",
+    "DuplicateOptionError",
+    "DuplicateSectionError",
+    "NoOptionError",
+    "NoConfigFileReadError",
+    "ParsingError",
+    "MissingSectionHeaderError",
+    "ConfigUpdater",
+    "Option",
+    "Section",
+    "Comment",
+    "Space",
+]
 
 
 class NoConfigFileReadError(Error):
     """Raised when no configuration file was read but update requested."""
+
     def __init__(self):
-        super().__init__(
-            "No configuration file was yet read! Use .read(...) first.")
+        super().__init__("No configuration file was yet read! Use .read(...) first.")
 
 
 # Used in parser getters to indicate the default behaviour when a specific
@@ -56,8 +73,8 @@ _UNSET = object()
 
 
 class Container(ABC):
-    """Abstract Mixin Class
-    """
+    """Abstract Mixin Class describing a container that holds blocks"""
+
     def __init__(self, **kwargs):
         self._structure = list()
         super().__init__(**kwargs)
@@ -67,12 +84,32 @@ class Container(ABC):
         return self._structure
 
     @property
-    def last_item(self):
+    def first_block(self):
+        if self._structure:
+            return self._structure[0]
+        else:
+            return None
+
+    @property
+    def last_block(self):
         if self._structure:
             return self._structure[-1]
         else:
             return None
 
+    def _remove_block(self, idx):
+        """Remove block at index idx within container
+
+        Use `.container_idx` of a block to get the index.
+        Not meant for users, rather use block.remove() instead!
+        """
+        del self._structure[idx]
+        return self
+
+    def __len__(self):
+        """"Number of blocks in container"""
+        return len(self._structure)
+
 
 class Block(ABC):
     """Abstract Block type holding lines
@@ -80,17 +117,15 @@ class Block(ABC):
     Block objects hold original lines from the configuration file and hold
     a reference to a container wherein the object resides.
     """
-    def __init__(self, container=None, **kwargs):
+
+    def __init__(self, container, **kwargs):
         self._container = container
-        self.lines = []
+        self._lines = []
         self._updated = False
         super().__init__(**kwargs)
 
     def __str__(self):
-        return ''.join(self.lines)
-
-    def __len__(self):
-        return len(self.lines)
+        return "".join(self._lines)
 
     def __eq__(self, other):
         if isinstance(other, self.__class__):
@@ -104,129 +139,81 @@ class Block(ABC):
         Args:
             line (str): one line to add
         """
-        self.lines.append(line)
+        self._lines.append(line)
         return self
 
+    @property
+    def lines(self):
+        return self._lines
+
     @property
     def container(self):
+        """"Returns current container holding the block"""
         return self._container
 
+    @property
+    def container_idx(self):
+        """"Returns index of the block within its container"""
+        return self._container.structure.index(self)
+
+    @property
+    def updated(self):
+        """Returns if the option was changed/updated"""
+        # if no lines were added, treat it as updated since we added it
+        return self._updated or not self._lines
+
     @property
     def add_before(self):
         """Returns a builder inserting a new block before the current block"""
-        idx = self._container.structure.index(self)
-        return BlockBuilder(self._container, idx)
+        return BlockBuilder(self._container, self.container_idx)
 
     @property
     def add_after(self):
         """Returns a builder inserting a new block after the current block"""
-        idx = self._container.structure.index(self)
-        return BlockBuilder(self._container, idx+1)
+        return BlockBuilder(self._container, self.container_idx + 1)
 
+    @property
+    def next_block(self):
+        """Returns the next block in the current container"""
+        idx = self.container_idx + 1
+        if idx < len(self._container):
+            return self._container.structure[idx]
+        else:
+            return None
 
-class BlockBuilder(object):
-    """Builder that injects blocks at a given index position."""
-    def __init__(self, container, idx):
-        self._container = container
-        self._idx = idx
+    @property
+    def previous_block(self):
+        """Returns the previous block in the current container"""
+        idx = self.container_idx - 1
+        if idx >= 0:
+            return self._container.structure[idx]
+        else:
+            return None
 
-    def comment(self, text, comment_prefix='#'):
-        """Creates a comment block
-
-        Args:
-            text (str): content of comment without #
-            comment_prefix (str): character indicating start of comment
-
-        Returns:
-            self for chaining
-        """
-        comment = Comment(self._container)
-        if not text.startswith(comment_prefix):
-            text = "{} {}".format(comment_prefix, text)
-        if not text.endswith('\n'):
-            text = "{}{}".format(text, '\n')
-        comment.add_line(text)
-        self._container.structure.insert(self._idx, comment)
-        self._idx += 1
-        return self
-
-    def section(self, section):
-        """Creates a section block
-
-        Args:
-            section (str or :class:`Section`): name of section or object
-
-        Returns:
-            self for chaining
-        """
-        if not isinstance(self._container, ConfigUpdater):
-            raise ValueError("Sections can only be added at section level!")
-        if isinstance(section, str):
-            # create a new section
-            section = Section(section, container=self._container)
-        elif not isinstance(section, Section):
-            raise ValueError("Parameter must be a string or Section type!")
-        if section.name in [block.name for block in self._container
-                            if isinstance(block, Section)]:
-            raise DuplicateSectionError(section.name)
-        self._container.structure.insert(self._idx, section)
-        self._idx += 1
-        return self
-
-    def space(self, newlines=1):
-        """Creates a vertical space of newlines
-
-        Args:
-            newlines (int): number of empty lines
-
-        Returns:
-            self for chaining
-        """
-        space = Space()
-        for line in range(newlines):
-            space.add_line('\n')
-        self._container.structure.insert(self._idx, space)
-        self._idx += 1
-        return self
-
-    def option(self, key, value=None, **kwargs):
-        """Creates a new option inside a section
-
-        Args:
-            key (str): key of the option
-            value (str or None): value of the option
-            **kwargs: are passed to the constructor of :class:`Option`
-
-        Returns:
-            self for chaining
-        """
-        if not isinstance(self._container, Section):
-            raise ValueError("Options can only be added inside a section!")
-        option = Option(key, value, container=self._container, **kwargs)
-        if option.key in self._container.options():
-            raise DuplicateOptionError(self._container.name, option.key)
-        option.value = value
-        self._container.structure.insert(self._idx, option)
-        self._idx += 1
+    def remove(self):
+        """Remove this block from container"""
+        self.container._remove_block(self.container_idx)
         return self
 
 
 class Comment(Block):
     """Comment block"""
-    def __init__(self, container=None):
+
+    def __init__(self, container):
         super().__init__(container=container)
 
     def __repr__(self):
-        return '<Comment>'
+        return "<Comment>"
 
 
 class Space(Block):
     """Vertical space block of new lines"""
-    def __init__(self, container=None):
+
+    def __init__(self, container):
         super().__init__(container=container)
 
     def __repr__(self):
-        return '<Space>'
+        return "<Space>"
 
 
 class Section(Block, Container, MutableMapping):
@@ -236,6 +223,7 @@ class Section(Block, Container, MutableMapping):
         name (str): name of the section
         updated (bool): indicates name change or a new section
     """
+
     def __init__(self, name, container, **kwargs):
         self._name = name
         self._structure = list()
@@ -261,10 +249,10 @@ class Section(Block, Container, MutableMapping):
         Args:
             line (str): one line in the comment
         """
-        if not isinstance(self.last_item, Comment):
-            comment = Comment(self._structure)
+        if not isinstance(self.last_block, Comment):
+            comment = Comment(container=self)
             self._structure.append(comment)
-        self.last_item.add_line(line)
+        self.last_block.add_line(line)
         return self
 
     def add_space(self, line):
@@ -275,15 +263,18 @@ class Section(Block, Container, MutableMapping):
         Args:
             line (str): one line that defines the space, maybe whitespaces
         """
-        if not isinstance(self.last_item, Space):
-            space = Space(self._structure)
+        if not isinstance(self.last_block, Space):
+            space = Space(container=self)
             self._structure.append(space)
-        self.last_item.add_line(line)
+        self.last_block.add_line(line)
         return self
 
     def _get_option_idx(self, key):
-        idx = [i for i, entry in enumerate(self._structure)
-               if isinstance(entry, Option) and entry.key == key]
+        idx = [
+            i
+            for i, entry in enumerate(self._structure)
+            if isinstance(entry, Option) and entry.key == key
+        ]
         if idx:
             return idx[0]
         else:
@@ -299,7 +290,7 @@ class Section(Block, Container, MutableMapping):
         return s
 
     def __repr__(self):
-        return '<Section: {}>'.format(self.name)
+        return "<Section: {}>".format(self.name)
 
     def __getitem__(self, key):
         key = self._container.optionxform(key)
@@ -325,17 +316,13 @@ class Section(Block, Container, MutableMapping):
     def __contains__(self, key):
         return key in self.options()
 
-    def __len__(self):
-        return len(self._structure)
-
     def __iter__(self):
         """Return all entries, not just options"""
         return self._structure.__iter__()
 
     def __eq__(self, other):
         if isinstance(other, self.__class__):
-            return (self.name == other.name and
-                    self._structure == other._structure)
+            return self.name == other.name and self._structure == other._structure
         else:
             return False
 
@@ -345,8 +332,7 @@ class Section(Block, Container, MutableMapping):
         Returns:
             list: list of :class:`Option` blocks
         """
-        return [entry for entry in self._structure
-                if isinstance(entry, Option)]
+        return [entry for entry in self._structure if isinstance(entry, Option)]
 
     def options(self):
         """Returns option names
@@ -364,12 +350,6 @@ class Section(Block, Container, MutableMapping):
         """
         return {key: self.__getitem__(key).value for key in self.options()}
 
-    @property
-    def updated(self):
-        """Returns if the option was changed/updated"""
-        # if no lines were added, treat it as updated since we added it
-        return self._updated or not self.lines
-
     @property
     def name(self):
         return self._name
@@ -419,8 +399,16 @@ class Option(Block):
         value (str): stored value
         updated (bool): indicates name change or a new section
     """
-    def __init__(self, key, value, container, delimiter='=',
-                 space_around_delimiters=True, line=None):
+
+    def __init__(
+        self,
+        key,
+        value,
+        container,
+        delimiter="=",
+        space_around_delimiters=True,
+        line=None,
+    ):
         super().__init__(container=container)
         self._key = key
         self._values = [value]
@@ -431,7 +419,7 @@ class Option(Block):
         self._multiline_value_joined = False
         self._space_around_delimiters = space_around_delimiters
         if line:
-            self.lines.append(line)
+            super().add_line(line)
 
     def add_line(self, line):
         super().add_line(line)
@@ -440,30 +428,24 @@ class Option(Block):
     def _join_multiline_value(self):
         if not self._multiline_value_joined and not self._value_is_none:
             # do what `_join_multiline_value` in ConfigParser would do
-            self._value = '\n'.join(self._values).rstrip()
+            self._value = "\n".join(self._values).rstrip()
             self._multiline_value_joined = True
 
     def __str__(self):
         if not self.updated:
             return super().__str__()
         if self._value is None:
-            return "{}{}".format(self._key, '\n')
+            return "{}{}".format(self._key, "\n")
         if self._space_around_delimiters:
             # no space is needed if we use multi-line arguments
-            suffix = '' if str(self._value).startswith('\n') else ' '
+            suffix = "" if str(self._value).startswith("\n") else " "
             delim = " {}{}".format(self._delimiter, suffix)
         else:
             delim = self._delimiter
-        return "{}{}{}{}".format(self._key, delim, self._value, '\n')
+        return "{}{}{}{}".format(self._key, delim, self._value, "\n")
 
     def __repr__(self):
-        return '<Option: {} = {}>'.format(self.key, self.value)
-
-    @property
-    def updated(self):
-        """Returns if the option was changed/updated"""
-        # if no lines were added, treat it as updated since we added it
-        return self._updated or not self.lines
+        return "<Option: {} = {}>".format(self.key, self.value)
 
     @property
     def key(self):
@@ -487,7 +469,7 @@ class Option(Block):
         self._value = value
         self._values = [value]
 
-    def set_values(self, values, separator='\n', indent=4*' '):
+    def set_values(self, values, separator="\n", indent=4 * " "):
         """Sets the value to a given list of options, e.g. multi-line values
 
         Args:
@@ -499,19 +481,107 @@ class Option(Block):
         self._updated = True
         self._multiline_value_joined = True
         self._values = values
-        if separator == '\n':
-            values = [''] + values
+        if separator == "\n":
+            values = [""] + values
             separator = separator + indent
         self._value = separator.join(values)
 
 
+class BlockBuilder(object):
+    """Builder that injects blocks at a given index position."""
+
+    def __init__(self, container, idx):
+        self._container = container
+        self._idx = idx
+
+    def comment(self, text, comment_prefix="#"):
+        """Creates a comment block
+
+        Args:
+            text (str): content of comment without #
+            comment_prefix (str): character indicating start of comment
+
+        Returns:
+            self for chaining
+        """
+        comment = Comment(self._container)
+        if not text.startswith(comment_prefix):
+            text = "{} {}".format(comment_prefix, text)
+        if not text.endswith("\n"):
+            text = "{}{}".format(text, "\n")
+        comment.add_line(text)
+        self._container.structure.insert(self._idx, comment)
+        self._idx += 1
+        return self
+
+    def section(self, section):
+        """Creates a section block
+
+        Args:
+            section (str or :class:`Section`): name of section or object
+
+        Returns:
+            self for chaining
+        """
+        if not isinstance(self._container, ConfigUpdater):
+            raise ValueError("Sections can only be added at section level!")
+        if isinstance(section, str):
+            # create a new section
+            section = Section(section, container=self._container)
+        elif not isinstance(section, Section):
+            raise ValueError("Parameter must be a string or Section type!")
+        if section.name in [
+            block.name for block in self._container if isinstance(block, Section)
+        ]:
+            raise DuplicateSectionError(section.name)
+        self._container.structure.insert(self._idx, section)
+        self._idx += 1
+        return self
+
+    def space(self, newlines=1):
+        """Creates a vertical space of newlines
+
+        Args:
+            newlines (int): number of empty lines
+
+        Returns:
+            self for chaining
+        """
+        space = Space(container=self._container)
+        for _ in range(newlines):
+            space.add_line("\n")
+        self._container.structure.insert(self._idx, space)
+        self._idx += 1
+        return self
+
+    def option(self, key, value=None, **kwargs):
+        """Creates a new option inside a section
+
+        Args:
+            key (str): key of the option
+            value (str or None): value of the option
+            **kwargs: are passed to the constructor of :class:`Option`
+
+        Returns:
+            self for chaining
+        """
+        if not isinstance(self._container, Section):
+            raise ValueError("Options can only be added inside a section!")
+        option = Option(key, value, container=self._container, **kwargs)
+        if option.key in self._container.options():
+            raise DuplicateOptionError(self._container.name, option.key)
+        option.value = value
+        self._container.structure.insert(self._idx, option)
+        self._idx += 1
+        return self
+
+
 class ConfigUpdater(Container, MutableMapping):
     """Parser for updating configuration files.
 
     ConfigUpdater follows the API of ConfigParser with some differences:
       * inline comments are treated as part of a key's value,
       * only a single config file can be updated at a time,
-      * empty lines in values are not valid,
       * the original case of sections and keys are kept,
       * control over the position of a new section/key.
 
@@ -523,6 +593,7 @@ class ConfigUpdater(Container, MutableMapping):
       * passing key/value-pairs with ``default`` argument,
       * non-strict mode allowing duplicate sections and keys.
     """
+
     # Regular expressions for parsing section headers and options
     _SECT_TMPL = r"""
         \[                                 # [
@@ -556,9 +627,17 @@ class ConfigUpdater(Container, MutableMapping):
     # Compiled regular expression for matching leading whitespace in a line
     NONSPACECRE = re.compile(r"\S")
 
-    def __init__(self, allow_no_value=False, *, delimiters=('=', ':'),
-                 comment_prefixes=('#', ';'), inline_comment_prefixes=None,
-                 strict=True, space_around_delimiters=True):
+    def __init__(
+        self,
+        allow_no_value=False,
+        *,
+        delimiters=("=", ":"),
+        comment_prefixes=("#", ";"),
+        inline_comment_prefixes=None,
+        strict=True,
+        empty_lines_in_values=True,
+        space_around_delimiters=True
+    ):
         """Constructor of ConfigUpdater
 
         Args:
@@ -569,6 +648,9 @@ class ConfigUpdater(Container, MutableMapping):
                 default None
             strict (bool): each section must be unique as well as every key
                 within a section, default True
+            empty_lines_in_values (bool): each empty line marks the end of an option.
+                Otherwise, internal empty lines of a multiline option are kept as part
+                of the value, default: True.
             space_around_delimiters (bool): add a space before and after the
                 delimiter, default True
         """
@@ -581,27 +663,27 @@ class ConfigUpdater(Container, MutableMapping):
         self._sections = self._dict()
         self._structure = []
         self._delimiters = tuple(delimiters)
-        if delimiters == ('=', ':'):
+        if delimiters == ("=", ":"):
             self._optcre = self.OPTCRE_NV if allow_no_value else self.OPTCRE
         else:
             d = "|".join(re.escape(d) for d in delimiters)
             if allow_no_value:
-                self._optcre = re.compile(self._OPT_NV_TMPL.format(delim=d),
-                                          re.VERBOSE)
+                self._optcre = re.compile(self._OPT_NV_TMPL.format(delim=d), re.VERBOSE)
             else:
-                self._optcre = re.compile(self._OPT_TMPL.format(delim=d),
-                                          re.VERBOSE)
+                self._optcre = re.compile(self._OPT_TMPL.format(delim=d), re.VERBOSE)
         self._comment_prefixes = tuple(comment_prefixes or ())
         self._inline_comment_prefixes = tuple(inline_comment_prefixes or ())
         self._strict = strict
         self._allow_no_value = allow_no_value
-        # Options from ConfigParser that we need to set constantly
-        self._empty_lines_in_values = False
+        self._empty_lines_in_values = empty_lines_in_values
         super().__init__()
 
     def _get_section_idx(self, name):
-        idx = [i for i, entry in enumerate(self._structure)
-               if isinstance(entry, Section) and entry.name == name]
+        idx = [
+            i
+            for i, entry in enumerate(self._structure)
+            if isinstance(entry, Section) and entry.name == name
+        ]
         if idx:
             return idx[0]
         else:
@@ -636,10 +718,10 @@ class ConfigUpdater(Container, MutableMapping):
             try:
                 source = f.name
             except AttributeError:
-                source = '<???>'
+                source = "<???>"
         self._read(f, source)
 
-    def read_string(self, string, source='<string>'):
+    def read_string(self, string, source="<string>"):
         """Read configuration from a given string.
 
         Args:
@@ -661,16 +743,16 @@ class ConfigUpdater(Container, MutableMapping):
         return optionstr.lower()
 
     def _update_curr_block(self, block_type):
-        if not isinstance(self.last_item, block_type):
+        if not isinstance(self.last_block, block_type):
             new_block = block_type(container=self)
             self._structure.append(new_block)
 
     def _add_comment(self, line):
-        if isinstance(self.last_item, Section):
-            self.last_item.add_comment(line)
+        if isinstance(self.last_block, Section):
+            self.last_block.add_comment(line)
         else:
             self._update_curr_block(Comment)
-            self.last_item.add_line(line)
+            self.last_block.add_line(line)
 
     def _add_section(self, sectname, line):
         new_section = Section(sectname, container=self)
@@ -679,19 +761,21 @@ class ConfigUpdater(Container, MutableMapping):
 
     def _add_option(self, key, vi, value, line):
         entry = Option(
-            key, value,
+            key,
+            value,
             delimiter=vi,
-            container=self.last_item,
+            container=self.last_block,
             space_around_delimiters=self._space_around_delimiters,
-            line=line)
-        self.last_item.add_option(entry)
+            line=line,
+        )
+        self.last_block.add_option(entry)
 
     def _add_space(self, line):
-        if isinstance(self.last_item, Section):
-            self.last_item.add_space(line)
+        if isinstance(self.last_block, Section):
+            self.last_block.add_space(line)
         else:
             self._update_curr_block(Space)
-            self.last_item.add_line(line)
+            self.last_block.add_line(line)
 
     def _read(self, fp, fpname):
         """Parse a sectioned configuration file.
@@ -716,12 +800,12 @@ class ConfigUpdater(Container, MutableMapping):
         """
         self._structure = []
         elements_added = set()
-        cursect = None                        # None, or a dictionary
+        cursect = None  # None, or a dictionary
         sectname = None
         optname = None
         lineno = 0
         indent_level = 0
-        e = None                              # None, or an exception
+        e = None  # None, or an exception
         for lineno, line in enumerate(fp, start=1):
             comment_start = sys.maxsize
             # strip inline comments
@@ -729,16 +813,17 @@ class ConfigUpdater(Container, MutableMapping):
             while comment_start == sys.maxsize and inline_prefixes:
                 next_prefixes = {}
                 for prefix, index in inline_prefixes.items():
-                    index = line.find(prefix, index+1)
+                    index = line.find(prefix, index + 1)
                     if index == -1:
                         continue
                     next_prefixes[prefix] = index
-                    if index == 0 or (index > 0 and line[index-1].isspace()):
+                    if index == 0 or (index > 0 and line[index - 1].isspace()):
                         comment_start = min(comment_start, index)
                 inline_prefixes = next_prefixes
             # strip full line comments
             for prefix in self._comment_prefixes:
-                if line.strip().startswith(prefix):
+                # configparser would do line.strip() here
+                if line.rstrip().startswith(prefix):
                     comment_start = 0
                     self._add_comment(line)  # HOOK
                     break
@@ -749,12 +834,15 @@ class ConfigUpdater(Container, MutableMapping):
                 if self._empty_lines_in_values:
                     # add empty line to the value, but only if there was no
                     # comment on the line
-                    if (comment_start is None and
-                            cursect is not None and
-                            optname and
-                            cursect[optname] is not None):
-                        cursect[optname].append('')  # newlines added at join
-                        self.last_item.last_item.add_line(line)  # HOOK
+                    if (
+                        comment_start is None
+                        and cursect is not None
+                        and optname
+                        and cursect[optname] is not None
+                    ):
+                        cursect[optname].append("")  # newlines added at join
+                        if line.strip():
+                            self.last_block.last_block.add_line(line)  # HOOK
                 else:
                     # empty line marks end of value
                     indent_level = sys.maxsize
@@ -764,21 +852,19 @@ class ConfigUpdater(Container, MutableMapping):
             # continuation line?
             first_nonspace = self.NONSPACECRE.search(line)
             cur_indent_level = first_nonspace.start() if first_nonspace else 0
-            if (cursect is not None and optname and
-                    cur_indent_level > indent_level):
+            if cursect is not None and optname and cur_indent_level > indent_level:
                 cursect[optname].append(value)
-                self.last_item.last_item.add_line(line)  # HOOK
+                self.last_block.last_block.add_line(line)  # HOOK
             # a section header or option header?
             else:
                 indent_level = cur_indent_level
                 # is it a section header?
                 mo = self.SECTCRE.match(value)
                 if mo:
-                    sectname = mo.group('header')
+                    sectname = mo.group("header")
                     if sectname in self._sections:
                         if self._strict and sectname in elements_added:
-                            raise DuplicateSectionError(sectname, fpname,
-                                                        lineno)
+                            raise DuplicateSectionError(sectname, fpname, lineno)
                         cursect = self._sections[sectname]
                         elements_added.add(sectname)
                     else:
@@ -795,16 +881,16 @@ class ConfigUpdater(Container, MutableMapping):
                 else:
                     mo = self._optcre.match(value)
                     if mo:
-                        optname, vi, optval = mo.group('option', 'vi', 'value')
+                        optname, vi, optval = mo.group("option", "vi", "value")
                         if not optname:
                             e = self._handle_error(e, fpname, lineno, line)
                         # optname = self.optionxform(optname.rstrip())
                         # keep original case of key
                         optname = optname.rstrip()
-                        if (self._strict and
-                                (sectname, optname) in elements_added):
-                            raise DuplicateOptionError(sectname, optname,
-                                                       fpname, lineno)
+                        if self._strict and (sectname, optname) in elements_added:
+                            raise DuplicateOptionError(
+                                sectname, optname, fpname, lineno
+                            )
                         elements_added.add((sectname, optname))
                         # This check is fine because the OPTCRE cannot
                         # match if it would set optval to None
@@ -824,6 +910,9 @@ class ConfigUpdater(Container, MutableMapping):
         # if any parsing errors occurred, raise an exception
         if e:
             raise e
+        # if empty_lines_in_values is true, we have to eliminate spurious newlines
+        if self._empty_lines_in_values:
+            self._check_values_with_blank_lines()
 
     def _handle_error(self, exc, fpname, lineno, line):
         if not exc:
@@ -831,6 +920,24 @@ class ConfigUpdater(Container, MutableMapping):
         exc.append(lineno, repr(line))
         return exc
 
+    def _check_values_with_blank_lines(self):
+        for section in self.section_blocks():
+            for option in section.option_blocks():
+                next_block = option.next_block
+                if isinstance(next_block, Space):
+                    # check if space is part of a multi-line value with blank lines
+                    if "".join(next_block.lines).strip():
+                        self._merge_option_with_space(option, next_block)
+
+    def _merge_option_with_space(self, option, space):
+        last_val_idx = max(i for i, line in enumerate(space.lines) if line.strip())
+        value_lines = space.lines[: last_val_idx + 1]
+        merge_vals = "".join(line.lstrip(" ") for line in value_lines)
+        option._values.append(merge_vals)
+        option._multiline_value_joined = False
+        option.lines.extend(space.lines[: last_val_idx + 1])
+        del space.lines[: last_val_idx + 1]
+
     def write(self, fp, validate=True):
         """Write an .ini-format representation of the configuration state.
 
@@ -852,7 +959,7 @@ class ConfigUpdater(Container, MutableMapping):
             raise NoConfigFileReadError()
         if validate:  # validate BEFORE opening the file!
             self.validate_format()
-        with open(self._filename, 'w') as fb:
+        with open(self._filename, "w") as fb:
             self.write(fb, validate=False)
 
     def validate_format(self, **kwargs):
@@ -866,21 +973,20 @@ class ConfigUpdater(Container, MutableMapping):
             allow_no_value=self._allow_no_value,
             inline_comment_prefixes=self._inline_comment_prefixes,
             strict=self._strict,
-            empty_lines_in_values=self._empty_lines_in_values
+            empty_lines_in_values=self._empty_lines_in_values,
         )
         args.update(kwargs)
         parser = ConfigParser(**args)
         updated_cfg = str(self)
         parser.read_string(updated_cfg)
 
-    def sections_blocks(self):
+    def section_blocks(self):
         """Returns all section blocks
 
         Returns:
             list: list of :class:`Section` blocks
         """
-        return [block for block in self._structure
-                if isinstance(block, Section)]
+        return [block for block in self._structure if isinstance(block, Section)]
 
     def sections(self):
         """Return a list of section names
@@ -888,13 +994,13 @@ class ConfigUpdater(Container, MutableMapping):
         Returns:
             list: list of section names
         """
-        return [section.name for section in self.sections_blocks()]
+        return [section.name for section in self.section_blocks()]
 
     def __str__(self):
-        return ''.join(str(block) for block in self._structure)
+        return "".join(str(block) for block in self._structure)
 
     def __getitem__(self, key):
-        for section in self.sections_blocks():
+        for section in self.section_blocks():
             if section.name == key:
                 return section
         else:
@@ -920,10 +1026,6 @@ class ConfigUpdater(Container, MutableMapping):
     def __contains__(self, key):
         return self.has_section(key)
 
-    def __len__(self):
-        """Number of all blocks, not just sections"""
-        return len(self._structure)
-
     def __iter__(self):
         """Iterate over all blocks, not just sections"""
         return self._structure.__iter__()
@@ -1014,7 +1116,7 @@ class ConfigUpdater(Container, MutableMapping):
             list: list of :class:`Section` or :class:`Option` objects
         """
         if section is _UNSET:
-            return [(sect.name, sect) for sect in self.sections_blocks()]
+            return [(sect.name, sect) for sect in self.section_blocks()]
 
         section = self.__getitem__(section)
         return [(opt.key, opt) for opt in section.option_blocks()]
@@ -1095,5 +1197,4 @@ class ConfigUpdater(Container, MutableMapping):
         Returns:
             dict: dictionary with same content
         """
-        return {sect: self.__getitem__(sect).to_dict()
-                for sect in self.sections()}
+        return {sect: self.__getitem__(sect).to_dict() for sect in self.sections()}
